def db_write_format():
    return {
        "type": "llm",
        "userId": "test",
        "message": "test",
        "timestamp": "test",
        "input": "test",
        "output": "test",
    }


def db_read_format():
    return {
        "type": "llm",
        "userId": "test",
        "message": "test",
        "timestamp": "test",
        "input": "test",
        "output": "test",
    }
